@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 XStream Committers.
+ * Copyright (C) 2017, 2018 XStream Committers.
  * All rights reserved.
  *
  * The software in this package is published under the terms of the BSD
@@ -26,7 +26,7 @@ public class JapaneseEraConverter extends AbstractSingleValueConverter {
 
     @Override
     public boolean canConvert(@SuppressWarnings("rawtypes") final Class type) {
-        return JapaneseEra.class.isAssignableFrom(type);
+        return type != null && JapaneseEra.class.isAssignableFrom(type);
     }
 
     @Override
